import os
import globus.manager.api.models as models


class VhostCleanupManager:
    """
    Class that checks for abandonded vhost configurations and
    unlinks them. Normally this is done automatically after collection
    deletion but if a DTN is offline for a long time, or there are parts of
    a previous endpoint left behind in that directory from a skipped cleanup,
    then there can be junk left around with dangling configuration info.
    """

    # Wait 5 minutes after start to check
    CHECK_DELAY = 60 * 5
    PRIORITY = 1

    def __init__(self, scheduler, app):
        self.scheduler = scheduler
        self.app = app
        self.scheduler.enter(self.CHECK_DELAY, self.PRIORITY, self.vhost_cleanup)
        self.vhost_dir = os.path.expanduser("~gcsweb/etc/httpd/conf.d/")

    def vhost_cleanup(self):
        """
        Remove vhost configs for deleted collections. This is useful if the DTN
        has been offline long enough for the app sync notification for
        collection deletion has expired.
        """
        with self.app.app_context():
            existing_collections = models.Collection.query.with_entities(
                models.Collection.id
            ).filter_by(deleted=False)
        valid_vhosts = list(map(lambda f: f.id + ".conf", existing_collections))

        for entry in os.listdir(self.vhost_dir):
            if entry not in valid_vhosts:
                entry_path = os.path.join(self.vhost_dir, entry)
                if not os.path.isdir(entry_path):
                    self.app.logger.debug(
                        "Removing " + entry + " from the vhosts config dir"
                    )
                    os.remove(entry_path)
